/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file ClassTest.h
 *    $Tests the class object$.
 *
 * @author Ronny Stricker
 * @date   2012/02/08
 */

#include <boost/test/unit_test.hpp>

#include <factory/NullClass.h>

#include "classes/DiamondClassStructure.h"

using namespace std;

namespace mira {

//////////////////////////////////////////////////////////////////////////////

//#############################################################################
//##  Class Operations Test
//#############################################################################

BOOST_AUTO_TEST_CASE( ClassOps )
{
	BOOST_CHECK( A::CLASS() != C::CLASS() );

	Object* tObject = C::CLASS().newInstance();
	BOOST_CHECK( C::CLASS() == tObject->getClass() );
	
	delete tObject;
	
	BOOST_CHECK_THROW( A::CLASS().getClassByIdentifier("UnknownClass"), mira::XFactoryUnknown );
	
	BOOST_CHECK_EQUAL( A::CLASS().getTypename(), "mira::A" );
}

BOOST_AUTO_TEST_CASE( NullClassTest )
{
	BOOST_CHECK_EQUAL( A::CLASS() == NullClass::null(), false );
	BOOST_CHECK_EQUAL( NullClass::null() == NullClass::null(), true );
}

BOOST_AUTO_TEST_CASE( PseudoClassOps )
{
	// check pseudo class newInstance functions
	Object* tObject = Object::CLASS().newInstance("mira::A");
	BOOST_REQUIRE( tObject != NULL );
	BOOST_CHECK_EQUAL( tObject->getClass().getIdentifier(), "mira::A" );
	delete tObject;

	tObject = Object::CLASS().newInstance<A>("mira::A");
	BOOST_REQUIRE( tObject != NULL );
	BOOST_CHECK_EQUAL( tObject->getClass().getIdentifier(), "mira::A" );
	delete tObject;
}

//////////////////////////////////////////////////////////////////////////////

} // namespace
