/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file RSASignatureTest.C
 *    Test case for RSASignature.
 *
 * @author Christian Martin
 * @date   2011/01/14
 */

#include <boost/test/unit_test.hpp>

#include <security/RSASignature.h>

#include <utils/ToString.h>

using namespace mira;
using namespace std;

///////////////////////////////////////////////////////////////////////////////

BOOST_AUTO_TEST_CASE( RSASignatureTest_MD5 )
{
	RSAKey tPublicKey, tPrivateKey;
	RSAKey::generateKey(1024, tPublicKey, tPrivateKey);

	std::string tMsg("This is a test message.");
	std::string tSignatureStr;

	{
		RSASignature tSignature =
			RSASignature::signMessage(tPrivateKey, RSASignature::DIGEST_MD5,
			                          tMsg.c_str(), tMsg.size());
		tSignatureStr = toString(tSignature);
	}

	bool tVerified = false;
	{
		RSASignature tSignature = fromString<RSASignature>(tSignatureStr);
		tVerified = 
			RSASignature::verifyMessage(tPublicKey, RSASignature::DIGEST_MD5,
			                            tMsg.c_str(), tMsg.size(), tSignature);
	}

	BOOST_CHECK(tVerified);
}

BOOST_AUTO_TEST_CASE( RSASignatureTest_SHA1 )
{
	RSAKey tPublicKey, tPrivateKey;
	RSAKey::generateKey(1024, tPublicKey, tPrivateKey);

	std::string tMsg("This is a test message.");
	std::string tSignatureStr;

	{
		RSASignature tSignature =
			RSASignature::signMessage(tPrivateKey, RSASignature::DIGEST_SHA1,
			                          tMsg.c_str(), tMsg.size());
		tSignatureStr = toString(tSignature);
	}

	bool tVerified = false;
	{
		RSASignature tSignature = fromString<RSASignature>(tSignatureStr);
		tVerified = 
			RSASignature::verifyMessage(tPublicKey, RSASignature::DIGEST_SHA1,
			                            tMsg.c_str(), tMsg.size(), tSignature);
	}

	BOOST_CHECK(tVerified);
}

BOOST_AUTO_TEST_CASE( RSASignatureTest_RIPEMD160 )
{
	RSAKey tPublicKey, tPrivateKey;
	RSAKey::generateKey(1024, tPublicKey, tPrivateKey);

	std::string tMsg("This is a test message.");
	std::string tSignatureStr;

	{
		RSASignature tSignature =
			RSASignature::signMessage(tPrivateKey, RSASignature::DIGEST_RIPEMD160,
			                          tMsg.c_str(), tMsg.size());
		tSignatureStr = toString(tSignature);
	}

	bool tVerified = false;
	{
		RSASignature tSignature = fromString<RSASignature>(tSignatureStr);
		tVerified = 
			RSASignature::verifyMessage(tPublicKey, RSASignature::DIGEST_RIPEMD160,
			                            tMsg.c_str(), tMsg.size(), tSignature);
	}

	BOOST_CHECK(tVerified);
}
